@@ -183,7 +183,7 @@ class AuthenticatedHandler(web.RequestHandler):
     def logged_in(self):
         """Is a user currently logged in?"""
         user = self.current_user
-        return user and not user == "anonymous"
+        return user and user != "anonymous"
 
     @property
     def login_handler(self):
@@ -682,7 +682,7 @@ class JupyterHandler(AuthenticatedHandler):
             try:
                 message = exception.log_message % exception.args
             except Exception:
-                pass
+                pass  # noqa
 
             # construct the custom reason, if defined
             reason = getattr(exception, "reason", "")
@@ -773,7 +773,9 @@ class APIHandler(JupyterHandler):
     def finish(self, *args, **kwargs):
         """Finish an API response."""
         self.update_api_activity()
-        self.set_header("Content-Type", "application/json")
+        # Allow caller to indicate content-type...
+        set_content_type = kwargs.pop("set_content_type", "application/json")
+        self.set_header("Content-Type", set_content_type)
         return super().finish(*args, **kwargs)
 
     def options(self, *args, **kwargs):
@@ -891,7 +893,7 @@ class AuthenticatedFileHandler(JupyterHandler, web.StaticFileHandler):
         abs_path = super().validate_absolute_path(root, absolute_path)
         abs_root = os.path.abspath(root)
         assert abs_path is not None
-        if is_hidden(abs_path, abs_root) and not self.contents_manager.allow_hidden:
+        if not self.contents_manager.allow_hidden and is_hidden(abs_path, abs_root):
             self.log.info(
                 "Refusing to serve hidden file, via 404 Error, use flag 'ContentsManager.allow_hidden' to enable"
             )
