@@ -5,11 +5,11 @@ import mimetypes
 from base64 import decodebytes
 from typing import List
 
+from jupyter_core.utils import ensure_async
 from tornado import web
 
 from jupyter_server.auth import authorized
 from jupyter_server.base.handlers import JupyterHandler
-from jupyter_server.utils import ensure_async
 
 AUTH_RESOURCE = "contents"
 
@@ -27,6 +27,7 @@ class FilesHandler(JupyterHandler, web.StaticFileHandler):
 
     @property
     def content_security_policy(self):
+        """The content security policy."""
         # In case we're serving HTML/SVG, confine any Javascript to a unique
         # origin so it can't interact with the notebook server.
         return super().content_security_policy + "; sandbox allow-scripts"
@@ -34,6 +35,7 @@ class FilesHandler(JupyterHandler, web.StaticFileHandler):
     @web.authenticated
     @authorized
     def head(self, path):
+        """The head response."""
         self.get(path, include_body=False)
         self.check_xsrf_cookie()
         return self.get(path, include_body=False)
@@ -41,11 +43,12 @@ class FilesHandler(JupyterHandler, web.StaticFileHandler):
     @web.authenticated
     @authorized
     async def get(self, path, include_body=True):
+        """Get a file by path."""
         # /files/ requests must originate from the same site
         self.check_xsrf_cookie()
         cm = self.contents_manager
 
-        if await ensure_async(cm.is_hidden(path)) and not cm.allow_hidden:
+        if not cm.allow_hidden and await ensure_async(cm.is_hidden(path)):
             self.log.info("Refusing to serve hidden file, via 404 Error")
             raise web.HTTPError(404)
 
