@@ -1,3 +1,4 @@
+"""Kernel gateway managers."""
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 import asyncio
@@ -10,12 +11,13 @@ from threading import Thread
 from time import monotonic
 from typing import Any, Dict, Optional
 
-import websocket
+import websocket  # type:ignore
 from jupyter_client.asynchronous.client import AsyncKernelClient
 from jupyter_client.clientabc import KernelClientABC
 from jupyter_client.kernelspec import KernelSpecManager
 from jupyter_client.manager import AsyncKernelManager
 from jupyter_client.managerabc import KernelManagerABC
+from jupyter_core.utils import ensure_async
 from tornado import web
 from tornado.escape import json_decode, json_encode, url_escape, utf8
 from traitlets import DottedObjectName, Instance, Type, default
@@ -23,7 +25,7 @@ from traitlets import DottedObjectName, Instance, Type, default
 from .._tz import UTC
 from ..services.kernels.kernelmanager import AsyncMappingKernelManager
 from ..services.sessions.sessionmanager import SessionManager
-from ..utils import ensure_async, url_path_join
+from ..utils import url_path_join
 from .gateway_client import GatewayClient, gateway_request
 
 
@@ -31,7 +33,7 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
     """Kernel manager that supports remote kernels hosted by Jupyter Kernel or Enterprise Gateway."""
 
     # We'll maintain our own set of kernel ids
-    _kernels: Dict[str, "GatewayKernelManager"] = {}
+    _kernels: Dict[str, "GatewayKernelManager"] = {}  # type:ignore[assignment]
 
     @default("kernel_manager_class")
     def _default_kernel_manager_class(self):
@@ -42,6 +44,7 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
         return False  # no need to share zmq contexts
 
     def __init__(self, **kwargs):
+        """Initialize a gateway mapping kernel manager."""
         super().__init__(**kwargs)
         self.kernels_url = url_path_join(
             GatewayClient.instance().url, GatewayClient.instance().kernels_endpoint
@@ -54,7 +57,7 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
         except KeyError:
             pass
 
-    async def start_kernel(self, kernel_id=None, path=None, **kwargs):
+    async def start_kernel(self, *, kernel_id=None, path=None, **kwargs):
         """Start a kernel for a session and return its kernel_id.
 
         Parameters
@@ -69,9 +72,8 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
         """
         self.log.info(f"Request start kernel: kernel_id={kernel_id}, path='{path}'")
 
-        if kernel_id is None:
-            if path is not None:
-                kwargs["cwd"] = self.cwd_for_path(path)
+        if kernel_id is None and path is not None:
+            kwargs["cwd"] = self.cwd_for_path(path)
 
         km = self.kernel_manager_factory(parent=self, log=self.log)
         await km.start_kernel(kernel_id=kernel_id, **kwargs)
@@ -96,7 +98,7 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
         model = None
         km = self.get_kernel(str(kernel_id))
         if km:
-            model = km.kernel
+            model = km.kernel  # type:ignore[attr-defined]
         return model
 
     async def list_kernels(self, **kwargs):
@@ -141,7 +143,7 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
             The purpose of this shutdown is to restart the kernel (True)
         """
         km = self.get_kernel(kernel_id)
-        await km.shutdown_kernel(now=now, restart=restart)
+        await ensure_async(km.shutdown_kernel(now=now, restart=restart))
         self.remove_kernel(kernel_id)
 
     async def restart_kernel(self, kernel_id, now=False, **kwargs):
@@ -153,7 +155,7 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
             The id of the kernel to restart.
         """
         km = self.get_kernel(kernel_id)
-        await km.restart_kernel(now=now, **kwargs)
+        await ensure_async(km.restart_kernel(now=now, **kwargs))
 
     async def interrupt_kernel(self, kernel_id, **kwargs):
         """Interrupt a kernel by its kernel uuid.
@@ -164,14 +166,14 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
             The id of the kernel to interrupt.
         """
         km = self.get_kernel(kernel_id)
-        await km.interrupt_kernel()
+        await ensure_async(km.interrupt_kernel())
 
     async def shutdown_all(self, now=False):
         """Shutdown all kernels."""
         kids = list(self._kernels)
         for kernel_id in kids:
             km = self.get_kernel(kernel_id)
-            await km.shutdown_kernel(now=now)
+            await ensure_async(km.shutdown_kernel(now=now))
             self.remove_kernel(kernel_id)
 
     async def cull_kernels(self):
@@ -181,7 +183,10 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
 
 
 class GatewayKernelSpecManager(KernelSpecManager):
+    """A gateway kernel spec manager."""
+
     def __init__(self, **kwargs):
+        """Initialize a gateway kernel spec manager."""
         super().__init__(**kwargs)
         base_endpoint = url_path_join(
             GatewayClient.instance().url, GatewayClient.instance().kernelspecs_endpoint
@@ -195,14 +200,37 @@ class GatewayKernelSpecManager(KernelSpecManager):
 
     @staticmethod
     def _get_endpoint_for_user_filter(default_endpoint):
+        """Get the endpoint for a user filter."""
         kernel_user = os.environ.get("KERNEL_USERNAME")
         if kernel_user:
             return "?user=".join([default_endpoint, kernel_user])
         return default_endpoint
 
+    def _replace_path_kernelspec_resources(self, kernel_specs):
+        """Helper method that replaces any gateway base_url with the server's base_url
+        This enables clients to properly route through jupyter_server to a gateway
+        for kernel resources such as logo files
+        """
+        kernelspecs = kernel_specs["kernelspecs"]
+        for kernel_name in kernelspecs:
+            resources = kernelspecs[kernel_name]["resources"]
+            for resource_name in resources:
+                original_path = resources[resource_name]
+                split_eg_base_url = str.rsplit(original_path, sep="/kernelspecs/", maxsplit=1)
+                if len(split_eg_base_url) > 1:
+                    new_path = url_path_join(
+                        self.parent.base_url, "kernelspecs", split_eg_base_url[1]
+                    )
+                    kernel_specs["kernelspecs"][kernel_name]["resources"][resource_name] = new_path
+                    if original_path != new_path:
+                        self.log.debug(
+                            f"Replaced original kernel resource path {original_path} with new "
+                            f"path {kernel_specs['kernelspecs'][kernel_name]['resources'][resource_name]}"
+                        )
+        return kernel_specs
+
     def _get_kernelspecs_endpoint_url(self, kernel_name=None):
         """Builds a url for the kernels endpoint
-
         Parameters
         ----------
         kernel_name : kernel name (optional)
@@ -213,6 +241,7 @@ class GatewayKernelSpecManager(KernelSpecManager):
         return self.base_endpoint
 
     async def get_all_specs(self):
+        """Get all of the kernel specs for the gateway."""
         fetched_kspecs = await self.list_kernel_specs()
 
         # get the default kernel name and compare to that of this server.
@@ -237,6 +266,7 @@ class GatewayKernelSpecManager(KernelSpecManager):
         self.log.debug(f"Request list kernel specs at: {kernel_spec_url}")
         response = await gateway_request(kernel_spec_url, method="GET")
         kernel_specs = json_decode(response.body)
+        kernel_specs = self._replace_path_kernelspec_resources(kernel_specs)
         return kernel_specs
 
     async def get_kernel_spec(self, kernel_name, **kwargs):
@@ -252,15 +282,11 @@ class GatewayKernelSpecManager(KernelSpecManager):
         try:
             response = await gateway_request(kernel_spec_url, method="GET")
         except web.HTTPError as error:
-            if error.status_code == 404:
+            if error.status_code == 404:  # noqa[PLR2004]
                 # Convert not found to KeyError since that's what the Notebook handler expects
                 # message is not used, but might as well make it useful for troubleshooting
-                raise KeyError(
-                    "kernelspec {kernel_name} not found on Gateway server at: {gateway_url}".format(
-                        kernel_name=kernel_name,
-                        gateway_url=GatewayClient.instance().url,
-                    )
-                ) from error
+                msg = f"kernelspec {kernel_name} not found on Gateway server at: {GatewayClient.instance().url}"
+                raise KeyError(msg) from None
             else:
                 raise
         else:
@@ -285,7 +311,7 @@ class GatewayKernelSpecManager(KernelSpecManager):
         try:
             response = await gateway_request(kernel_spec_resource_url, method="GET")
         except web.HTTPError as error:
-            if error.status_code == 404:
+            if error.status_code == 404:  # noqa[PLR2004]
                 kernel_spec_resource = None
             else:
                 raise
@@ -295,26 +321,31 @@ class GatewayKernelSpecManager(KernelSpecManager):
 
 
 class GatewaySessionManager(SessionManager):
+    """A gateway session manager."""
+
     kernel_manager = Instance("jupyter_server.gateway.managers.GatewayMappingKernelManager")
 
-    async def kernel_culled(self, kernel_id):
+    async def kernel_culled(self, kernel_id: str) -> bool:  # typing: ignore
         """Checks if the kernel is still considered alive and returns true if it's not found."""
-        kernel = None
+        km: Optional[GatewayKernelManager] = None
         try:
+            # Since we keep the models up-to-date via client polling, use that state to determine
+            # if this kernel no longer exists on the gateway server rather than perform a redundant
+            # fetch operation - especially since this is called at approximately the same interval.
+            # This has the effect of reducing GET /api/kernels requests against the gateway server
+            # by 50%!
+            # Note that should the redundant polling be consolidated, or replaced with an event-based
+            # notification model, this will need to be revisited.
             km = self.kernel_manager.get_kernel(kernel_id)
-            kernel = await km.refresh_model()
         except Exception:  # Let exceptions here reflect culled kernel
-            pass
-        return kernel is None
-
-
-"""KernelManager class to manage a kernel running on a Gateway Server via the REST API"""
+            pass  # noqa
+        return km is None
 
 
 class GatewayKernelManager(AsyncKernelManager):
     """Manages a single kernel remotely via a Gateway Server."""
 
-    kernel_id = None
+    kernel_id: Optional[str] = None  # type:ignore[assignment]
     kernel = None
 
     @default("cache_ports")
@@ -322,11 +353,13 @@ class GatewayKernelManager(AsyncKernelManager):
         return False  # no need to cache ports here
 
     def __init__(self, **kwargs):
+        """Initialize the gateway kernel manager."""
         super().__init__(**kwargs)
         self.kernels_url = url_path_join(
             GatewayClient.instance().url, GatewayClient.instance().kernels_endpoint
         )
-        self.kernel_url = self.kernel = self.kernel_id = None
+        self.kernel_url: str
+        self.kernel = self.kernel_id = None
         # simulate busy/activity markers:
         self.execution_state = self.last_activity = None
 
@@ -344,13 +377,13 @@ class GatewayKernelManager(AsyncKernelManager):
 
     def client(self, **kwargs):
         """Create a client configured to connect to our kernel"""
-        kw = {}
+        kw: dict = {}
         kw.update(self.get_connection_info(session=True))
         kw.update(
-            dict(
-                connection_file=self.connection_file,
-                parent=self,
-            )
+            {
+                "connection_file": self.connection_file,
+                "parent": self,
+            }
         )
         kw["kernel_id"] = self.kernel_id
 
@@ -373,7 +406,7 @@ class GatewayKernelManager(AsyncKernelManager):
                 response = await gateway_request(self.kernel_url, method="GET")
 
             except web.HTTPError as error:
-                if error.status_code == 404:
+                if error.status_code == 404:  # noqa[PLR2004]
                     self.log.warning("Kernel not found at: %s" % self.kernel_url)
                     model = None
                 else:
@@ -420,22 +453,28 @@ class GatewayKernelManager(AsyncKernelManager):
             if os.environ.get("KERNEL_USERNAME") is None and GatewayClient.instance().http_user:
                 os.environ["KERNEL_USERNAME"] = GatewayClient.instance().http_user
 
+            payload_envs = os.environ.copy()
+            payload_envs.update(kwargs.get("env", {}))  # Add any env entries in this request
+
+            # Build the actual env payload, filtering allowed_envs and those starting with 'KERNEL_'
             kernel_env = {
                 k: v
-                for (k, v) in dict(os.environ).items()
-                if k.startswith("KERNEL_") or k in GatewayClient.instance().env_whitelist.split(",")
+                for (k, v) in payload_envs.items()
+                if k.startswith("KERNEL_") or k in GatewayClient.instance().allowed_envs.split(",")
             }
 
-            # Add any env entries in this request
-            kernel_env.update(kwargs.get("env", {}))
-
             # Convey the full path to where this notebook file is located.
             if kwargs.get("cwd") is not None and kernel_env.get("KERNEL_WORKING_DIR") is None:
                 kernel_env["KERNEL_WORKING_DIR"] = kwargs["cwd"]
 
             json_body = json_encode({"name": kernel_name, "env": kernel_env})
 
-            response = await gateway_request(self.kernels_url, method="POST", body=json_body)
+            response = await gateway_request(
+                self.kernels_url,
+                method="POST",
+                headers={"Content-Type": "application/json"},
+                body=json_body,
+            )
             self.kernel = json_decode(response.body)
             self.kernel_id = self.kernel["id"]
             self.kernel_url = url_path_join(self.kernels_url, url_escape(str(self.kernel_id)))
@@ -455,7 +494,7 @@ class GatewayKernelManager(AsyncKernelManager):
                 response = await gateway_request(self.kernel_url, method="DELETE")
                 self.log.debug("Shutdown kernel response: %d %s", response.code, response.reason)
             except web.HTTPError as error:
-                if error.status_code == 404:
+                if error.status_code == 404:  # noqa[PLR2004]
                     self.log.debug("Shutdown kernel response: kernel not found (ignored)")
                 else:
                     raise
@@ -466,7 +505,12 @@ class GatewayKernelManager(AsyncKernelManager):
             assert self.kernel_url is not None
             kernel_url = self.kernel_url + "/restart"
             self.log.debug("Request restart kernel at: %s", kernel_url)
-            response = await gateway_request(kernel_url, method="POST", body=json_encode({}))
+            response = await gateway_request(
+                kernel_url,
+                method="POST",
+                headers={"Content-Type": "application/json"},
+                body=json_encode({}),
+            )
             self.log.debug("Restart kernel response: %d %s", response.code, response.reason)
 
     async def interrupt_kernel(self):
@@ -475,7 +519,12 @@ class GatewayKernelManager(AsyncKernelManager):
             assert self.kernel_url is not None
             kernel_url = self.kernel_url + "/interrupt"
             self.log.debug("Request interrupt kernel at: %s", kernel_url)
-            response = await gateway_request(kernel_url, method="POST", body=json_encode({}))
+            response = await gateway_request(
+                kernel_url,
+                method="POST",
+                headers={"Content-Type": "application/json"},
+                body=json_encode({}),
+            )
             self.log.debug("Interrupt kernel response: %d %s", response.code, response.reason)
 
     async def is_alive(self):
@@ -496,11 +545,13 @@ KernelManagerABC.register(GatewayKernelManager)
 
 
 class ChannelQueue(Queue):
+    """A queue for a named channel."""
 
     channel_name: Optional[str] = None
     response_router_finished: bool
 
     def __init__(self, channel_name: str, channel_socket: websocket.WebSocket, log: Logger):
+        """Initialize a channel queue."""
         super().__init__()
         self.channel_name = channel_name
         self.channel_socket = channel_socket
@@ -508,10 +559,12 @@ class ChannelQueue(Queue):
         self.response_router_finished = False
 
     async def _async_get(self, timeout=None):
+        """Asynchronously get from the queue."""
         if timeout is None:
             timeout = float("inf")
         elif timeout < 0:
-            raise ValueError("'timeout' must be a non-negative number")
+            msg = "'timeout' must be a non-negative number"
+            raise ValueError(msg)
         end_time = monotonic() + timeout
 
         while True:
@@ -519,12 +572,14 @@ class ChannelQueue(Queue):
                 return self.get(block=False)
             except Empty:
                 if self.response_router_finished:
-                    raise RuntimeError("Response router had finished")
+                    msg = "Response router had finished"
+                    raise RuntimeError(msg) from None
                 if monotonic() > end_time:
                     raise
                 await asyncio.sleep(0)
 
     async def get_msg(self, *args: Any, **kwargs: Any) -> dict:
+        """Get a message from the queue."""
         timeout = kwargs.get("timeout", 1)
         msg = await self._async_get(timeout=timeout)
         self.log.debug(
@@ -536,6 +591,7 @@ class ChannelQueue(Queue):
         return msg
 
     def send(self, msg: dict) -> None:
+        """Send a message to the queue."""
         message = json.dumps(msg, default=ChannelQueue.serialize_datetime).replace("</", "<\\/")
         self.log.debug(
             "Sending message on channel: {}, msg_id: {}, msg_type: {}".format(
@@ -546,14 +602,17 @@ class ChannelQueue(Queue):
 
     @staticmethod
     def serialize_datetime(dt):
+        """Serialize a datetime object."""
         if isinstance(dt, datetime.datetime):
             return dt.timestamp()
         return None
 
     def start(self) -> None:
+        """Start the queue."""
         pass
 
     def stop(self) -> None:
+        """Stop the queue."""
         if not self.empty():
             # If unprocessed messages are detected, drain the queue collecting non-status
             # messages.  If any remain that are not 'shutdown_reply' and this is not iopub
@@ -573,11 +632,15 @@ class ChannelQueue(Queue):
                 )
 
     def is_alive(self) -> bool:
+        """Whether the queue is alive."""
         return self.channel_socket is not None
 
 
 class HBChannelQueue(ChannelQueue):
+    """A queue for the hearbeat channel."""
+
     def is_beating(self) -> bool:
+        """Whether the channel is beating."""
         # Just use the is_alive status for now
         return self.is_alive()
 
@@ -603,13 +666,14 @@ class GatewayKernelClient(AsyncKernelClient):
     allow_stdin = False
     _channels_stopped: bool
     _channel_queues: Optional[Dict[str, ChannelQueue]]
-    _control_channel: Optional[ChannelQueue]
-    _hb_channel: Optional[ChannelQueue]
-    _stdin_channel: Optional[ChannelQueue]
-    _iopub_channel: Optional[ChannelQueue]
-    _shell_channel: Optional[ChannelQueue]
+    _control_channel: Optional[ChannelQueue]  # type:ignore[assignment]
+    _hb_channel: Optional[ChannelQueue]  # type:ignore[assignment]
+    _stdin_channel: Optional[ChannelQueue]  # type:ignore[assignment]
+    _iopub_channel: Optional[ChannelQueue]  # type:ignore[assignment]
+    _shell_channel: Optional[ChannelQueue]  # type:ignore[assignment]
 
     def __init__(self, kernel_id, **kwargs):
+        """Initialize a gateway kernel client."""
         super().__init__(**kwargs)
         self.kernel_id = kernel_id
         self.channel_socket: Optional[websocket.WebSocket] = None
