@@ -1,3 +1,4 @@
+"""Custom trait types."""
 import inspect
 from ast import literal_eval
 
@@ -44,11 +45,13 @@ class TypeFromClasses(ClassBasedTraitType):
 
         # OneOfType requires a list of klasses to be specified (different than Type).
         if not isinstance(klasses, (list, tuple, set)):
-            raise TraitError("`klasses` must be a list of class names (type is str) or classes.")
+            msg = "`klasses` must be a list of class names (type is str) or classes."
+            raise TraitError(msg)
 
         for klass in klasses:
             if not (inspect.isclass(klass) or isinstance(klass, str)):
-                raise TraitError("A OneOfType trait must specify a list of classes.")
+                msg = "A OneOfType trait must specify a list of classes."
+                raise TraitError(msg)
 
         # Store classes.
         self.klasses = klasses
@@ -65,15 +68,16 @@ class TypeFromClasses(ClassBasedTraitType):
             try:
                 value = self._resolve_string(value)
             except ImportError as e:
-                raise TraitError(
-                    "The '%s' trait of %s instance must be a type, but "
-                    "%r could not be imported" % (self.name, obj, value)
-                ) from e
+                emsg = (
+                    "The '{}' trait of {} instance must be a type, but "
+                    "{!r} could not be imported".format(self.name, obj, value)
+                )
+                raise TraitError(emsg) from e
         try:
             if self.subclass_from_klasses(value):
                 return value
         except Exception:
-            pass
+            pass  # noqa
 
         self.error(obj, value)
 
@@ -85,17 +89,18 @@ class TypeFromClasses(ClassBasedTraitType):
                 klass = klass.__module__ + "." + klass.__name__
             result += f"{klass} or "
         # Strip the last "or"
-        result = result.strip(" or ")  # noqa B005
+        result = result.strip(" or ")  # noqa
         if self.allow_none:
             return result + " or None"
         return result
 
     def instance_init(self, obj):
+        """Initialize an instance."""
         self._resolve_classes()
         super().instance_init(obj)
 
     def _resolve_classes(self):
-        # Resolve all string names to actual classes.
+        """Resolve all string names to actual classes."""
         self.importable_klasses = []
         for klass in self.klasses:
             if isinstance(klass, str):
@@ -104,7 +109,7 @@ class TypeFromClasses(ClassBasedTraitType):
                     klass = self._resolve_string(klass)
                     self.importable_klasses.append(klass)
                 except Exception:
-                    pass
+                    pass  # noqa
             else:
                 self.importable_klasses.append(klass)
 
@@ -112,6 +117,7 @@ class TypeFromClasses(ClassBasedTraitType):
             self.default_value = self._resolve_string(self.default_value)
 
     def default_value_repr(self):
+        """The default value repr."""
         value = self.default_value
         if isinstance(value, str):
             return repr(value)
@@ -162,9 +168,11 @@ class InstanceFromClasses(ClassBasedTraitType):
             )
 
         if (kw is not None) and not isinstance(kw, dict):
-            raise TraitError("The 'kw' argument must be a dict or None.")
+            msg = "The 'kw' argument must be a dict or None."
+            raise TraitError(msg)
         if (args is not None) and not isinstance(args, tuple):
-            raise TraitError("The 'args' argument must be a tuple or None.")
+            msg = "The 'args' argument must be a tuple or None."
+            raise TraitError(msg)
 
         self.default_args = args
         self.default_kwargs = kw
@@ -176,12 +184,14 @@ class InstanceFromClasses(ClassBasedTraitType):
         return any(isinstance(value, klass) for klass in self.importable_klasses)
 
     def validate(self, obj, value):
+        """Validate an instance."""
         if self.instance_from_importable_klasses(value):
             return value
         else:
             self.error(obj, value)
 
     def info(self):
+        """Get the trait info."""
         result = "an instance of "
         assert self.klasses is not None
         for klass in self.klasses:
@@ -190,17 +200,18 @@ class InstanceFromClasses(ClassBasedTraitType):
             else:
                 result += describe("a", klass)
             result += " or "
-        result = result.strip(" or ")  # noqa B005
+        result = result.strip(" or ")  # noqa
         if self.allow_none:
             result += " or None"
         return result
 
     def instance_init(self, obj):
+        """Initialize the trait."""
         self._resolve_classes()
         super().instance_init(obj)
 
     def _resolve_classes(self):
-        # Resolve all string names to actual classes.
+        """Resolve all string names to actual classes."""
         self.importable_klasses = []
         assert self.klasses is not None
         for klass in self.klasses:
@@ -210,11 +221,12 @@ class InstanceFromClasses(ClassBasedTraitType):
                     klass = self._resolve_string(klass)
                     self.importable_klasses.append(klass)
                 except Exception:
-                    pass
+                    pass  # noqa
             else:
                 self.importable_klasses.append(klass)
 
     def make_dynamic_default(self):
+        """Make the dynamic default for the trait."""
         if (self.default_args is None) and (self.default_kwargs is None):
             return None
         return self.klass(  # type:ignore[attr-defined]
@@ -222,7 +234,9 @@ class InstanceFromClasses(ClassBasedTraitType):
         )
 
     def default_value_repr(self):
+        """Get the default value repr."""
         return repr(self.make_dynamic_default())
 
     def from_string(self, s):
+        """Convert from a string."""
         return literal_eval(s)
