@@ -313,7 +313,9 @@ def is_namespace_package(namespace):
     # NOTE: using submodule_search_locations because the loader can be None
     try:
         spec = importlib.util.find_spec(namespace)
-    except ValueError:  # spec is not set - see https://docs.python.org/3/library/importlib.html#importlib.util.find_spec
+    except (
+        ValueError
+    ):  # spec is not set - see https://docs.python.org/3/library/importlib.html#importlib.util.find_spec
         return None
 
     if not spec:
@@ -412,7 +414,7 @@ def import_item(name):
     """
 
     parts = name.rsplit(".", 1)
-    if len(parts) == 2:
+    if len(parts) == 2:  # noqa
         # called with 'foo.bar....'
         package, obj = parts
         module = __import__(package, fromlist=[obj])
